#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Apr  4 10:20:25 2022

@author: rapa
"""
import numpy as np
import matplotlib.pyplot as plt

def invertedGaussians(xv):
    """
    Multiple-well inverted Gaussian function in 3D

    Args:
        xv (1D numpy array): x=xv[0], y=xv[1]
        
    Returns:
        f(x,y) (float): value of the function at (x,y)
    """
    # location of the wells
    xW = np.array([[0.3, 0.25], [0.25, 0.8], [0.75, 0.75]])
    # steepness of the wells (smaller means smaller stddev, means steeper)
    sigma = np.array([0.2,0.2,0.2])
    # amplitudes of the wells
    A = np.array([1,1,1.2])
    f = 0.
    for i in np.arange(len(xW)):
        f -= A[i] * np.exp(-(np.linalg.norm(xv - xW[i,:]))**2 / (2*sigma[i]**2))
    return f


if __name__ == '__main__':
    N = 101
    xx = np.linspace(0.,1.,N)
    yy = np.linspace(0.,1.,N)
    
    # Loop over all (x,y) points and evaluation objective function
    f = np.zeros((N,N))
    xv = np.zeros(2)
    for i in range(N):
        for j in range(N):
            xv[0] = xx[i]
            xv[1] = yy[j]
            f[i,j] = invertedGaussians(xv)
            
    # Plot contours.  Include contour labels.  
    fig, axs = plt.subplots()
    Cf = axs.contour(xx,yy,f.transpose())
    axs.clabel(Cf)
    axs.set_xlabel('x')
    axs.set_ylabel('y')
    axs.grid(True)
    axs.axis('square')
    plt.show()

    # Plot the surface.
    from matplotlib import cm
    fig, ax = plt.subplots(subplot_kw={"projection": "3d"})


    surf = ax.plot_surface(xx,yy,f.transpose(), cmap=cm.coolwarm,
                           linewidth=1, antialiased=True, rstride=3,cstride = 3)
    wireframe = ax.plot_wireframe(xx,yy,f.transpose(), cmap=cm.coolwarm,
                           linewidth=0.4, antialiased=True, rstride=3,cstride = 3)
    #contour3d = ax.contour3D(xx,yy,f.transpose(), 50, cmap=cm.coolwarm)

#     x = np.arange(1, 10)
# y = x.reshape(-1, 1)
# h = x * y

# cs = plt.contourf(h, levels=[10, 30, 50],
#     colors=['#808080', '#A0A0A0', '#C0C0C0'], extend='both')
# cs.cmap.set_over('red')
# cs.cmap.set_under('blue')
# cs.changed()

# from matplotlib import cm
# from matplotlib.ticker import LinearLocator

# fig, ax = plt.subplots(subplot_kw={"projection": "3d"})

# # Make data.
# X = np.arange(-5, 5, 0.25)
# Y = np.arange(-5, 5, 0.25)
# X, Y = np.meshgrid(X, Y)
# R = np.sqrt(X**2 + Y**2)
# Z = np.sin(R)

# # Plot the surface.
# #surf = ax.plot_surface(X, Y, Z, cmap=cm.coolwarm,
# #                       linewidth=0, antialiased=False)

# x, y = np.meshgrid(x,y)
# h = x * y
# surf = ax.plot_surface(x, y, h, cmap=cm.coolwarm,
#                        linewidth=0, antialiased=False)

# # Customize the z axis.
# ax.set_zlim(0, 10)
# ax.zaxis.set_major_locator(LinearLocator(10))
# # A StrMethodFormatter is used automatically
# ax.zaxis.set_major_formatter('{x:.02f}')

# # Add a color bar which maps values to colors.
# fig.colorbar(surf, shrink=0.5, aspect=5)

# plt.show()

