"""
Discover the exponential: a power law that equals its derivative
"""


import numpy as np
import matplotlib.pyplot as plt




############################################
# manually test some values of c
############################################

def run_test0(N):
    
    x = np.linspace(0,1,N)
    h = x[1]-x[0]

    c = 2.8
    f = c**x
    dfdx = (c**(x+h) - f)/h
    
    plt.close('all')
    fig = plt.figure()
    plt.plot(x, f, 'r', label='f')
    plt.plot(x, dfdx, 'b', label='df/dx')
    plt.xlabel('x')
    plt.grid(alpha=0.25)
    plt.legend(loc='lower right')
    plt.show()
    

    
def funcJ(c,x):

    h = x[1]-x[0]
    cxi = c**x
    cxiplus1 = c**(x+h)
    J = np.sum( (cxi - (cxiplus1 - cxi)/h)**2 ) / N
    xcxi = x*cxi	       # element-wise product
    xcxiplus1 = (x+h)*cxiplus1
    Jprime = 2*np.sum( (xcxi - (xcxiplus1 - xcxi)/h) * (cxi - (cxiplus1 - cxi)/h)) / N
    
    return J, Jprime


############################################
# Shape of the objective function
############################################

def run_test1(N): 
    
    x = np.linspace(0,1,N)
    Js = []
    cs = np.linspace(0.01,5,100)
    
    for c in cs:
        
        J, Jprime = funcJ(c,x)
        Js.append(J)
        
    
    plt.close('all')
    fig = plt.figure()
    plt.plot(cs, Js, 'b', label='J(c)')
    plt.xlabel('c')
    plt.grid(alpha=0.25)
    plt.legend(loc='lower right')
    plt.savefig("Jc.pdf")
    plt.show()
    

############################################
# optimize over c to match f and dfdx
############################################
    
def run_test2(N):    

    x = np.linspace(0,1,N)
    h = x[1]-x[0]

    c = 2.0
    J, Jprime = funcJ(c,x)
    alpha = 0.4         # 0.4 is good
    tol = 1e-8
    ii = 0
    
    while J > tol and ii < 50:
    
        ii = ii + 1
        print(c)
        c = c - alpha*Jprime
        J, Jprime = funcJ(c,x)
        
    return c

############################################
# main script
############################################
    
if __name__ == "__main__":
    
    N = int(1e3)
    
    run_test0()
    run_test1(N)
    
    c = run_test2(N)
    print(c)
    
    